package dico.modele;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.TreeMap;

/**
 * Un dictionnaire, liant un mot à <em>des</em> définitions.
 * 
 * <p>
 * Moralement, cette classe devrait être une interface, avec une implantation
 * cachée.
 * 
 * <p> Pour créer un nouveau dictionnaire, il faut passer par la méthode <code>creerDictionnaire</code>.
 * <p> On utilise un pattern pour créer le dictionnaire : Factory Method.
 * Une méthode est chargée de créer un objet (souvent d'une classe différente).
 * 
 * <p> Dans une première version, j'avais fait deux classes : DictionnaireFactory et Dictionnaire, 
 * mais pour cet exercice c'était quand même prendre un marteau pilon pour écraser une mouche. 
 *
 * <p> Autre Remarque : on a bien fait attention, dans les méthodes qui permettent de
 * consulter le dictionnaire, que celles-ci ne permettent pas de modifications
 * du contenu de celui-ci.
 * <p>
 * C'est une bonne idée, quand on renvoie une collection, de vérifier que les
 * modifications éventuelles du contenu de celle-ci n'auraient pas de
 * conséquence.
 * 
 * @author rosmord
 * 
 */
public class Dictionnaire {

	private TreeMap<String, EntreeDico> representation = new TreeMap<String, EntreeDico>();

	public static Dictionnaire creerDictionnaire() {
		// On cache que le dictionnaire est lu depuis un fichier
		// (ce qui est le boulot de lecteur dictionnaire).
		// Comme ça, si on décide un jour de changer d'implantation (passer à
		// une BD, par exemple)
		// Ce sera transparent pour le reste du code.
		LecteurDictionnaire lecteurDictionnaire = new LecteurDictionnaire();
		return lecteurDictionnaire.lireDictionnaire();
	}

	/**
	 * Pas public : c'est le "lecteur dictionnaire" qui saura le construire.
	 */
	Dictionnaire() {
	}

	/**
	 * Récupère les définitions pour un mot.
	 * <p>
	 * Si le mot n'est pas dans le dictionnaire, renvoie la liste vide.
	 * 
	 * @param mot
	 *            le mot dont on cherche la définition.
	 * @return une liste (non modifiable) de définitions.
	 */
	public EntreeDico getEntree(String mot) {
		EntreeDico resultat = representation.get(mot);
		if (resultat == null) {
			List<String> vide= Collections.emptyList();
			return new EntreeDico(mot, vide);
		} else {
			//
			return resultat;
		}
	}

	/**
	 * Liste des entrées dans le dictionnaire. (affichable, par exemple, dans
	 * une JList).
	 * 
	 * @return
	 */
	public List<String> getEntrees() {
		return new ArrayList<String>(representation.keySet());
	}

	/**
	 * Méthode non publique : notre dictionnaire n'est pas modifiable.
	 */

	void ajouterDefinition(String entree, String definition) {
		// Soyons paranos :
		if (entree == null || definition == null)
			throw new NullPointerException();
		if ("".equals(entree.trim()) || "".equals(definition.trim()))
			throw new IllegalArgumentException("Chaine vide interdite");
		// ok
		EntreeDico entreeDico = representation.get(entree);
		if (entreeDico == null) {
			entreeDico= new EntreeDico(entree);
			representation.put(entree, entreeDico);
		}
		entreeDico.ajouterDefinition(definition);

	}
}
