package dico.modele;

import java.io.InputStream;

import javax.xml.parsers.*;

import org.xml.sax.Attributes;
import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Lecteur du dictionnaire.
 * <p>
 * Complètement hors programme : le dictionnaire récupéré est en XML, avec une
 * structure assez complexe.
 * <p>
 * Normalement, pour lire ce genre de choses, j'utilise une bibliothèque comme
 * JDom. Mais là, je veux me contenter des bibliothèques standards, ce qui
 * complique un peu le code.
 * <p>
 * Comme la structure du dictionnaire récupéré est beaucoup plus complexe que ce
 * que je veux, je l'analyse en récupérant uniquement : les balises
 * <code>&lt;orth&gt;</code>, qui sont l'orthographe des mots, et les balises
 * <code>&lt;def&gt;</code> qui sont des textes de définition. Quand il y a
 * plusieurs orthographes, seule la dernière est prise en compte.
 * 
 * <p>
 * On utilise SAX, mais, encore une fois, c'est hors programme.
 * <p>
 * La classe n'est pas publique, parce que dans ce programme, je veux que seul
 * le "Dictionnaire" puisse l'appeler.
 * 
 * @author rosmord
 */
class LecteurDictionnaire {

	/**
	 * Classe auxiliaire (interne) utilisée pour l'analyse. pas visible de
	 * l'extérieur.
	 * 
	 * @author rosmord
	 */
	private class MyHandler extends DefaultHandler {
		// On se rapelle où on est :
		private static final int NULLE_PART = 0;
		private static final int DANS_ORTH = 1;
		private static final int DANS_DEF = 2;

		private int etat = NULLE_PART;
		String mot;

		@Override
		/**
		 * Appelé quand on rencontre une balise ouvrante.
		 * Le nom de la balise est dans localName. 
		 */
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) throws SAXException {
			// On stocke dans état le type de balise dans laquelle on est.
			if ("orth".equals(qName)) {
				// Orthographe d'une entrée
				etat = DANS_ORTH;
				mot= "";
			} else if ("def".equals(qName)) {
				// On entre dans une définition.
				etat = DANS_DEF;
			}
		}

		@Override
		// Appelé quand on rencontre une balise fermante.
		public void endElement(String uri, String localName, String qName)
				throws SAXException {
			// Sauf si je me trompe, orth et def ne peuvent contenir d'autre
			// balise.
			// à vérifier un de ces jours.
			etat = NULLE_PART;
		}

		@Override
		// On rencontre du texte.
		// celui-ci est une sous-chaîne de ch (à cause de problème d'optimisation du code).
		public void characters(char[] ch, int start, int length)
				throws SAXException {
			// Il y a plein de texte qu'on néglige (la phonétique, etc.)
			if (etat != NULLE_PART) {
				// Chic, le bon constructeur existe !
				String text= new String(ch, start, length);
				if (etat == DANS_ORTH) {
					mot= text;
				} else {
					// S'il y a un mot courant, lui ajouter la définition
					// (si elle n'est pas vide)
					if (null!= mot && ! "".equals(mot)&& ! "".equals(text.trim())) {
						dictionnaire.ajouterDefinition(mot, text);
					}
				}
			}
			
		}
	}

	private Dictionnaire dictionnaire = new Dictionnaire();

	public Dictionnaire lireDictionnaire() {
		InputStream in = this.getClass().getResourceAsStream("fra-eng.tei");
		SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
		saxParserFactory.setValidating(false);
		// parserFactory.setFeature("http://xml.org/sax/features/namespaces",
		// true);
		// parserFactory.setFeature(
		// "http://xml.org/sax/features/namespace-prefixes", false);
		try {
			SAXParser parser = saxParserFactory.newSAXParser();
			parser.parse(in, new MyHandler());
		} catch (Exception e) {
			// On ne rattrapera rien. Le dictionnaire n'a
			// qu'à être correct, d'abord.
			throw new RuntimeException(e);
		}
		return dictionnaire;
	}

}
